import cv2 as cv
import numpy as np


def debug(image):
    cv.imshow("debug", image)
    cv.waitKey(0)
def debug_contours(contours, image):
    mask = np.zeros((image.shape[0], image.shape[1]))
    cv.drawContours(mask, contours, -1, (255), 1)
    cv.imshow("debug", mask)
    cv.waitKey(0)

class BlackWhiteTransform:
    # For iterations with kmeans
    def __init__(self, percentage_led_area = 0.01, nb_color = 4, max_iter = 1, accuracy = 1):
        self.percetage_led_area = percentage_led_area
        self.nb_color = nb_color
        self.max_iter = max_iter
        self.accuracy = accuracy

    def convert_to_black_white(self, image, already_grayscale = False):
        # Transform the image into grascale, then use kmeans to group the different shades of
        # Grey

        min_area = image.shape[0] * image.shape[1] * self.percetage_led_area

        grey = cv.cvtColor(image, cv.COLOR_BGR2GRAY)
        shape = grey.shape

        grey = grey.reshape(grey.shape[0] * grey.shape[1], 1)
        grey = np.float32(grey)
    
        # Criteria is the number of iterations
        # Format : (type, max_iteration, accuracy)
        criteria = (cv.TERM_CRITERIA_EPS + cv.TERM_CRITERIA_MAX_ITER,\
            self.max_iter, self.accuracy)

        ret,label,center=cv.kmeans(grey,self.nb_color,None,\
            criteria,1,cv.KMEANS_RANDOM_CENTERS)

        center = np.uint8(center)
        res = center[label.flatten()]
        kmeans = res.reshape(shape)

        black_white = cv.threshold(kmeans, np.average(grey), 255, cv.THRESH_BINARY)[1]
        return black_white