import cv2 as cv
import numpy as np
from edge_detector import EdgeDetector
from black_and_white import BlackWhiteTransform
import logging

logger = logging.getLogger(__name__)

def debug(image):
    cv.imshow("debug", image)
    cv.waitKey(0)

class HSVColorDetector:

    def __init__(self, red = 180, blue = 45, min_gray = 60):
        self.red = red
        self.blue = blue
        self.min_gray = min_gray

    def detect_color(self, image, contours, shape, is_hsv = False):
        # WARNING: greyscale is NOT the same as the value in HSV
        # the formula for BGR to grayscale is 
        #           Gray  = 0.299*Red + 0.587*Green + 0.114*Blue
        # the formula for BGR to HSV (v = value similar to grayscale)
        #           Value = MAX(Red, Blue, Green)
        greyscale = cv.cvtColor(image, cv.COLOR_BGR2GRAY)
        if not is_hsv:
            image = cv.cvtColor(image, cv.COLOR_BGR2HLS_FULL)
        
        # for now the v is pretty much useless
        # cant really determine color intensity with it
        hue, saturation, _ = cv.split(image)
        # note : do not use sine or cosine for this, its like 10x slower than
        # just doing adding or substracting
        hue = np.abs(180 - hue) # due to angles looping back
        mask = np.zeros(shape)
        mask = cv.drawContours(mask, contours, -1, (255), 1)
        hue_mask = np.ma.array(data=hue, mask=mask)
        sat_mask = np.ma.array(data=saturation, mask=mask)

        huevalue = hue_mask.mean()
        satvalue = sat_mask.mean()
        print(huevalue)
        print(satvalue)
        if (abs(huevalue - self.blue) < abs(huevalue - self.red)):
            print("this is probably blue")
        else:
            print("this is probably red")
        
        if satvalue < self.min_gray:
            print("this is probably gray")
        else:
            print("this is probably not gray")
        

if __name__ == '__main__':
    logger.setLevel("DEBUG")
    image = cv.imread("test11.PNG")
    bwt = BlackWhiteTransform()
    black_white_image = bwt.convert_to_black_white(image)
    ed = EdgeDetector()
    edges = ed.get_edges(black_white_image)
    hsv = HSVColorDetector()
    hsv.detect_color(image, edges, black_white_image.shape)

